open Ast

let parse (filename:string) : Ast.cmd =
  let chan = open_in filename in
  let buf = Lexing.from_channel chan in
  try
    let ast = Parser.program Lexer.lex buf in
    close_in chan;
    ast
  with Parser.Error ->
    let open Lexing in
    failwith @@ Printf.sprintf "Parse error at (line:%d, column:%d)"
      (lexeme_start_p buf).pos_lnum
      ((lexeme_start_p buf).pos_cnum - (lexeme_start_p buf).pos_bol + 1)

let () =
  let source : Ast.cmd = parse Sys.argv.(1) in
  let target : X86.prog = Compile.compile source in
  print_endline (X86.string_of_prog target)
