%{
  (* Header: everything betweeen %{ and %} is exported directly to the
     beginning of the generated .ml file *)
  open Ast
%}

/* Begin grammatical specification */

/* Typically, we start by declaring tokens.  menhir uses the tokens to
   automatically generate a token datatype */
%token <string> IDENT
%token <int64> INT
%token EQ
%token SEMI
%token IFLZ
%token ELSE
%token WHILELZ
%token LPAREN
%token RPAREN
%token LBRACE
%token RBRACE
%token ADD
%token MUL
%token NEG
%token RETURN
%token EOF


/* Declare "program" as the starting nonterminal of the grammar */
%start program

/* Type annotations for program */
%type <Ast.cmd> program

%% /* Productions go here, after %% */

program:
  | c=command_list EOF  { c }

command_list:
  | c1=command c2=command_list { Seq (c1, c2) }
  | c=command { c }

command:
 | lhs=IDENT EQ rhs=exp SEMI { Assn (lhs, rhs) }
 | RETURN ret=exp SEMI       { Return ret }
 | IFLZ LPAREN cond=exp RPAREN LBRACE bthen=command_list RBRACE ELSE LBRACE belse=command_list RBRACE
                             { IfLZ (cond, bthen, belse) }
 | WHILELZ LPAREN cond=exp RPAREN LBRACE body=command_list RBRACE
                             { WhileLZ (cond, body) }


exp:
 | left=term ADD right=exp { Add (left, right) }
 | NEG e=exp               { Neg e }
 | t=term                  { t }

term:
 | left=factor MUL right=term { Mul (left, right) }
 | f=factor                   { f }

factor:      
 | var=IDENT           { Var var }
 | k=INT               { Const k }
 | LPAREN e=exp RPAREN { e }

